#!/usr/bin/env python
# -*- coding: utf-8 -*-

from flask import Flask
from flask import Response

from flask.ext.iniconfig import INIConfig

app = Flask(__name__)
INIConfig(app)
app.config.from_inifile('test.ini')

@app.route('/')
def index():
    def generate():
        for key, value in app.config.items():
            yield '%s: %s\n' % (key, value)
    return Response(generate(), mimetype='text/plain')

if __name__ == '__main__':
    app.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import ast
from ConfigParser import SafeConfigParser
from ConfigParser import _default_dict

from flask import current_app

class INIConfig(SafeConfigParser):

    def __init__(self, app=None, defaults=None, dict_type=_default_dict,
            allow_no_value=False):
        SafeConfigParser.__init__(self, defaults=defaults,
                dict_type=dict_type,
                allow_no_value=allow_no_value)
        # so the the normalization to lower case does not happen
        self.optionxform = str
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.app_config = app.config
        self.app_config.from_inifile = self.from_inifile
        self.app_config.from_inifile_sections = self.from_inifile_sections

    def from_inifile(self, path):
        self.read(path)
        for section in self.sections():
            options = self.options(section)
            for option in options:
                parsed_value = self.parse_value(section, option)
                if section == 'flask':
                    # flask likes its vars uppercase
                    self.app_config[option.upper()] = parsed_value
                else:
                    self.app_config.setdefault(section, {})[option] = parsed_value

    def from_inifile_sections(self, path, section_list, preserve_case=False):
        section_list.append('flask')
        section_list = set(section_list)
        self.read(path)
        for section in self.sections():
            options = self.options(section)
            for option in options:
                parsed_value = self.parse_value(section, option)
                if section in section_list:
                    if section == 'flask':
                        # flask likes its vars uppercase
                        self.app_config[option.upper()] = parsed_value
                    else:
                        if not preserve_case:
                            option = option.upper()
                        self.app_config[option] = parsed_value

    def parse_value(self, section, option):
        # XXX: deviates from SafeConfigParser: 1|0 evaluate to int
        for method in [self.getint, self.getfloat, self.getboolean]:
            try:
                return method(section, option)
            except ValueError:
                pass

        value = self.get(section, option).strip()
        try:
            # maybe its a dict, list or tuple
            if value and value[0] in [ '[', '{', '(' ]:
                return ast.literal_eval(value)
        except (ValueError, SyntaxError):
            pass

        return value



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from flask import Flask
# implicit test ??
from flask.ext.iniconfig import INIConfig

import nose.tools as nt

class TestINIConfig(object):

    @classmethod
    def setupClass(cls):
        cls.app = Flask('foo')
        INIConfig(cls.app)

    def setup(self):
        self.app.config.from_inifile('test.ini')

    @staticmethod
    def check_type(value, type_):
        nt.assert_is_instance(value, type_)

    @staticmethod
    def check_membership(config, name):
        nt.assert_in(name, config)

    @staticmethod
    def check_no_membership(config, name):
        nt.assert_not_in(name, config)

    def test_init_app(self):
        app = Flask('bar')
        INIConfig().init_app(app)

    def test_configparser_options(self):
        app = Flask('bar')
        INIConfig(defaults={'a': 1}, dict_type=dict,
                allow_no_value=True).init_app(app)

    def test_config_method(self):
        nt.ok_(hasattr(self.app.config, 'from_inifile'))

    def test_load_nofile_noerror(self):
        self.app.config.from_inifile('nonexistant.ini')

    def test_load_flask_default(self):
        nt.ok_('DEBUG' in self.app.config)
        nt.ok_(self.app.config['DEBUG'])

    def test_load_section_case_sensitivity(self):
        settings = self.app.config.get('INTS')
        nt.assert_is_none(settings)

    def test_load_property_case_sensitivity(self):
        settings = self.app.config.get('ints')
        property = settings.get('G')
        nt.assert_is_none(property)

    def test_load_basic_types(self):
        for type_ in [int, float, bool, str]:
            settings = self.app.config['%ss' % type_.__name__]
            for key, value in settings.items():
                callable = self.check_type
                callable.description = '%s_%s_%s' % (
                        self.test_load_basic_types.__name__, type_.__name__, key)
                yield callable, value, type_

    def test_load_json_list(self):
        settings = self.app.config['literals']
        nt.assert_is_instance(settings['o'], list)

    def test_load_json_dict(self):
        settings = self.app.config['literals']
        for name in ['p', 'q']:
            callable = self.check_type
            callable.description = '%s_%s' % (
                    self.test_load_json_dict.__name__, name)
            yield callable, settings[name], dict

    def test_load_json_embedded_list(self):
        settings = self.app.config['literals']
        nt.assert_is_instance(settings['q']['b'], list)

    def test_load_json_embedded_dict(self):
        settings = self.app.config['literals']
        nt.assert_is_instance(settings['q']['c'], dict)

    def test_load_json_tuple(self):
        settings = self.app.config['literals']
        nt.assert_is_instance(settings['r'], tuple)

    def test_load_colon_separated(self):
        settings = self.app.config['colonsep']
        for name in ['v', 'w']:
            callable = self.check_membership
            callable.description = '%s_%s' % (
                    self.test_load_colon_separated.__name__, name)
            yield callable, settings, name

    def test_load_empty(self):
        settings = self.app.config['empty']
        nt.eq_(settings['x'], '')

    def test_load_settings_only(self):
        app = Flask('bar')
        INIConfig().init_app(app)
        app.config.from_inifile_sections('test.ini', ['strs'])

        for name in ['DEBUG', 'S', 'T', 'U']:
            callable = self.check_membership
            callable.description = '%s_%s' % (
                    self.test_load_settings_only.__name__, name)
            yield callable, app.config, name

        for name in ['debug', 's', 't', 'u']:
            callable = self.check_no_membership
            callable.description = '%s_%s' % (
                    self.test_load_settings_only.__name__, name)
            yield callable, app.config, name

    def test_load_settings_only_with_preserve_case(self):
        app = Flask('bar')
        INIConfig().init_app(app)
        app.config.from_inifile_sections('test.ini', ['strs'],
                preserve_case=True)

        for name in ['DEBUG', 's', 't', 'u']:
            callable = self.check_membership
            callable.description = '%s_%s' % (
                    self.test_load_settings_only_with_preserve_case.__name__, name)
            yield callable, app.config, name

        for name in ['debug', 'S', 'T', 'U']:
            callable = self.check_no_membership
            callable.description = '%s_%s' % (
                    self.test_load_settings_only_with_preserve_case.__name__, name)
            yield callable, app.config, name



#! /usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

setup(name='Flask-INIConfig',
      version='0.0.4',
      author='Wampeter Foma',
      author_email='foma@wampeter.org',
      url='https://bitbucket.org/wampeter/flask-iniconfig/',
      license='BSD',
      description='A flask extension to load ini files via config',
      long_description=open('README.rst').read(),
      zip_safe=False,
      py_modules=['flask_iniconfig'],
      platforms='any',
      install_requires=['Flask'],
      setup_requires=['nose'],
      tests_require=['nose', 'coverage'],
      classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
